(
	local du = DeadlineUtil--this is the interface exposed by the Lightning Plug-in which provides communication between Deadline and 3ds Max
	if du == undefined do--if the script is not being run on Deadline (for testing purposes),
	(
		struct DeadlineUtilStruct --define a stand-in struct with the same methods as the Lightning plug-in
		(
			fn SetTitle title = ( format "Title: %\n" title ),
			fn SetProgress percent = (true),
			fn FailRender msg = ( throw msg ),
			fn GetSubmitInfoEntry key = ( undefined ),
			fn GetSubmitInfoEntryElementCount key = ( 0 ),
			fn GetSubmitInfoEntryElement index key = ( undefined ),
			fn GetJobInfoEntry key = ( undefined ),
			fn GetAuxFilename index = ( undefined ),
			fn GetOutputFilename index = ( undefined ),
			fn LogMessage msg = ( format "%\n" msg ),
			fn WarnMessage msg = ( format "Warning: %\n" msg ),
			CurrentFrame = ((sliderTime as string) as integer),
			CurrentTask = ( -1 ),
			SceneFileName = ( maxFilePath + maxFileName ),
			SceneFilePath = ( maxFilePath ),
			JobsDataFolder = ( "" ),
			PluginsFolder = ( "" )
		)
		du = DeadlineUtilStruct() --create an instance of the stand-in struct
	)--end if
	
	du.SetTitle "Bake To Texture (2021+)" --set the job title 
	du.LogMessage ">Starting Bake To Texture Job..." --output a message to the log
	local st = timestamp() --get the current system time
	
	BakeToTexture.outputPath = du.GetPathMappedFilename BakeToTexture.outputPath
	local bakeMode = execute (du.GetJobInfoEntry "BakeMode")
	local RenderTimeMode = execute (du.GetJobInfoEntry "RenderTimeMode")
	local totalToRender = execute (du.GetJobInfoEntry "TotalFramesToRender")
	local RenderTimeString = du.GetJobInfoEntry "RenderTimeString"
	local theFrame = du.CurrentFrame --this is the map index
	
	local allMaps = BakeToTexture.getAllMaps()
	
	du.LogMessage (">Output Path: "  + BakeToTexture.outputPath)
	du.LogMessage ("  >Objects: "  + (BakeToTexture.getObjects()).count as string)
	du.LogMessage ("  >Total Maps: "  + (BakeToTexture.getAllMaps()).count as string)
	local activeMaps = for a in allMaps where a.bakeable and a.renderable collect a
	du.LogMessage ("  >Active Maps: "  + activeMaps.count as string)
	
	local totalFramesToRender = 0	
	case bakeMode of
	(
		1: (
			for i = 1 to activeMaps.count where i != theFrame do activeMaps[i].renderable = false --disable all active maps but the one matching the frame index
			local theMap = activeMaps[theFrame]
			if RenderTimeMode == 1 then
			(
				du.LogMessage ("    >Rendering Map: " + theMap.fileName + "."+ theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
			)
			else
			(
				du.LogMessage ("    >Rendering Map: " + theMap.fileName + "####." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
			)
			totalFramesToRender = totalToRender
		)
		2: (
			for i = 1 to activeMaps.count where i != theFrame do activeMaps[i].renderable = false --disable all active maps but the one matching the frame index
			local theMap = activeMaps[theFrame]
			if RenderTimeMode == 1 then
			(
				if not matchPattern theMap.fileName pattern:"*0000" do theMap.fileName += "0000"
				du.LogMessage ("    >Rendering Map: " + theMap.fileName + "." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
			)
			else
			(
				du.LogMessage ("    >Rendering Map: " + theMap.fileName + "####." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
			)
			totalFramesToRender = totalToRender
		)	
		3: (
			du.LogMessage ("    >Rendering Maps: ")
			for theMap in activeMaps do 
			(
				totalFramesToRender += totalToRender
				if RenderTimeMode == 1 then
				(
					du.LogMessage ("      " + theMap.fileName + "." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
				)
				else
				(
					du.LogMessage ("      " + theMap.fileName + "####." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
				)
			)
		)
		4: (
			du.LogMessage ("    >Rendering Maps: ")
			for theMap in activeMaps do 
			(
				if RenderTimeMode == 1 then
				(
					local padding = (formattedPrint theFrame format:"04i")
					if not matchPattern theMap.fileName pattern:("*"+padding) do theMap.fileName += padding
					du.LogMessage ("      " + theMap.fileName + "." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
				)
				else
				(
					du.LogMessage ("      " + theMap.fileName + "####." + theMap.fileType + " @ " + theMap.imageWidth as string + "x" + theMap.imageHeight as string)
				)
				totalFramesToRender += totalToRender
			)
		)
	)
	du.LogMessage ("    >" + RenderTimeString ) 
	du.LogMessage ("    >Number Of Frames To Render Per Map: " + totalToRender as string)
		
	local totalRendered = #()
	fn callbackFN arg1 arg2 = 
	(
		append totalRendered arg2
		if arg1 then 
			du.LogMessage ( "  +Successfully Baked ["+ arg2 +"] - "+((100.0*totalRendered.count / totalFramesToRender) as string)+"%...")
		else
			du.LogMessage ( "  --Failed to Bake ["+ arg2 +"].")
		du.SetProgress (100.0*totalRendered.count / activeMaps.count)
		arg1
	)	
		
	BakeToTexture.bake onMapBakedCallback:callbackFN
	
	for i in activeMaps do i.renderable = true --restore the renderable state of the active maps
	if totalRendered.count == totalFramesToRender then
		du.LogMessage ("+Successfully Baked "+ totalRendered.count as string +" in " + ((timestamp() - st)/1000.0) as string + " sec.") --output the task duration
	else 
	(
		du.LogMessage ("--Expected Maps: " + totalFramesToRender as string + " - Baked Maps: " + totalRendered.count as string + " ")
		du.LogMessage ("--Failed To Bake To Texture after " + ((timestamp() - st)/1000.0) as string + " sec.") --output the task duration
	)
	totalRendered.count == totalFramesToRender --return true if the number of baked images matches the number of frames to render
)--end script