(
	global SMTDWorkflow_BakeToTexture2021plus
	global SMTD_Bake2021JobOptionsRollout
	global SMTD_BakeToTexture2021Rollout
	
	rollout SMTD_BakeToTexture2021Rollout "Bake to Texture"
	(
		button btn_update "UPDATE From Bake To Texture Dialog..."  width:230 offset:[0,0] align:#left across:2 --tooltip:"Add the selected object in the scene to the list of objects to bake." offset:[-10,-3] across:3
		button btn_openB2Tfloater "OPEN the Bake To Texture Dialog..."  width:230 offset:[0,0] align:#right --tooltip:"Add the selected object in the scene to the list of objects to bake." offset:[-10,-3] across:3
		label lbl_info "" offset:[0,3] align:#center
		group "Bake Mode:"
		(
			radiobuttons rad_bakeMode labels:#("ONE MAP PER TASK - If baking a Single Frame, use FRAME 0, NO Padding", "ONE MAP PER TASK - If baking a Single Frame, use PADDED Frame 0000","ALL MAPS IN ONE TASK - If baking a Single Frame, use Current Time, NO Padding","ALL MAPS IN ONE TASK - If baking a Single Frame, use PADDED Current Time") offset:[0,3] align:#left offsets:#([0,0],[0,5],[0,10],[0,15])
		)
	   
		fn refreshBakeUI = 
		(    
			rad_bakeMode.state = ::SMTDWorkflow_BakeToTexture2021plus.BakeMode
			local ObjectsToBake = BakeToTexture.getObjects()			
			local theMaps = BakeToTexture.getAllMaps()
			local theActiveMaps = for m in theMaps where m.renderable collect m
			local txt = "" as stringStream
			format "% Object% with % out of % Map% set to bake." (ObjectsToBake.count as string) (if ObjectsToBake.count == 1 then "" else "s") theActiveMaps.count theMaps.count (if theMaps.count == 1 then "" else "s") to:txt 
			lbl_info.text = txt as string
			try(SMTD_MainRollout.Ui_report ("+"+lbl_info.text))catch()
			try(SMTD_MainRollout.updateSubmitButtonState())catch()							--Update the SUBMIT button's enabled state and text
		)
		
		on btn_openB2Tfloater pressed do
		(
			BakeToTexture.showDialog()
		)

		on btn_update pressed do 
		(
			refreshBakeUI()
		)
		
		on rad_bakeMode changed itm do
		(
			::SMTDWorkflow_BakeToTexture2021plus.BakeMode = itm
			::SMTDWorkflow_BakeToTexture2021plus.storeSettings()
		)
		
		fn updateRolloutTitle state =
		(
			local theTitle = "Bake To Texture"
			if not state do
			(
				case ::SMTDWorkflow_BakeToTexture2021plus.BakeMode of
				(
					1: theTitle += " [One Map/Task (0 No Padding)]"
					2: theTitle += " [One Map/Task (0000 Padded)]"
					3: theTitle += " [1 Task (Single - No Padding)]"
					4: theTitle += " [1 Task (Single - Current Time Padded)]"
				)
			)
			SMTD_BakeToTexture2021Rollout.title = theTitle
		)
		
		on SMTD_BakeToTexture2021Rollout rolledup state do
		(
			updateRolloutTitle state
		)		
		
		on SMTD_BakeToTexture2021Rollout open do
		(
			::SMTDWorkflow_BakeToTexture2021plus.restoreSettings()
			refreshBakeUI()
			updateRolloutTitle SMTD_BakeToTexture2021Rollout.open
		)
		
		on SMTD_BakeToTexture2021Rollout close do
		(
			::SMTDWorkflow_BakeToTexture2021plus.storeSettings()
		)
	)--end bake to texture rollout
	
	local SMTDWorkflow_BakeToTexture2021plusStruct
	struct SMTDWorkflow_BakeToTexture2021plusStruct
	(
		fn isWorkflowActive = ((maxVersion())[1] >= 2300),	--Mandatory! Setting to false to disable the workflow in SMTD UI when 3ds Max 2020 and earlier is detected.
		WorkflowName = "baketexture2021plus", --internal name of the workflow
		WorkflowTitle= "Submit BAKE TO TEXTURE (3ds Max 2021+) Job To Deadline",	--the description of the workflow displayed in the drop-down list
		Description = "Submit a Bake To Texture job to be processed on Deadline Workers. Uses the feature introduced in 3ds Max 2021.", --the detailed description used for tooltips
		BakeMode = 1, --1: map per task 2: all maps in one task
		fn defineUILayout = 
		(
			#(
				#(#("Bake Job","Job Settings"),#(#(SMTD_BakeToTexture2021Rollout, true), #(SMTD_JobOptions,true), #(SMTD_JobFailureDetection,false))), 
				#(#("Assets","Assets - Asset Collection And Synchronization"),#(#(SMTD_AssetList,true))),
				#(#("Render","3ds Max Rendering"),#(#(SMTD_ScenePreProcessing,true), #(SMTD_MaxStartup,false), #(SMTD_MaxRendering,false), #(SMTD_Gamma,false))), 
				#(#("Options","Submission Options"),#(#(SMTD_UserOptions,true), #(SMTD_RendererExport,false), #(SMTD_SubmissionTimeouts,false))), 
				#(#("Limits","Black List Management and License Limits"),#(#(SMTD_BlackListSlaves,true), #(SMTD_LimitGroups,false)))
			)		
		),
		fn updateUILayout =
		(
			local disableArray = #(#btn_chunk, #btn_chunk, #spn_chunkSize, #ddl_OutOfOrderMethod, #btn_printFrameSequenceToLog, #spn_outOfOrderStep, #chk_forceWorkstationMode, #chk_useSilentMode, #chk_EnforceSequentialRendering )
			for c in SMTD_JobOptions.controls where findItem disableArray (c.name as name) > 0 do 
			(
				c.enabled = false
				try(c.tooltip = "Not available in this Workflow")catch()
			)			
		),
		fn storeSettings =
		(
			SMTDFunctions.storeWorkflowSettingsWithScene &SMTDWorkflow_BakeToTexture2021plus
		),
		fn restoreSettings =
		(
			SMTDFunctions.restoreWorkflowSettingsFromScene &SMTDWorkflow_BakeToTexture2021plus propsToSkip:#() 
		),				
		fn canSubmit =
		(
			local canSubmitJob = true																			--We start by assuming that a submission is possible, then we will lower the flag if something is not right
			local txt = "SUBMIT a BAKE TO TEXTURE Job to Deadline..."
			local ObjectsToBake = BakeToTexture.getObjects()
			if not renderers.current.supportsTexureBaking do													--report if the renderer does not support texture baking (e.g. Krakatoa)
			(
				txt = "The current renderer ["+(classof renderers.current) as string +"] does not support Texture Baking!"
				canSubmitJob = false
			)
			if ObjectsToBake.count == 0 do																		--If no objects were enabled for submission,
			(
				txt = "No Objects for Texture Baking!\n\nPlease set up at least one object via the Bake To Texture dialog!"
				canSubmitJob = false
			)
			if not BakeToTexture.bakeableMapsAreValid() do
			(
				txt = "Some Maps cannot be baked!\n\nPlease open the Bake To Texture dialog and make sure there are no Error messages when pressing the Bake button."
				canSubmitJob = false			
			)
			local theActiveMaps = for m in BakeToTexture.getAllMaps() where m.renderable collect m				--collect all active maps
			if theActiveMaps.count == 0 do
			(
				txt = "No Maps are active!\n\nPlease open the Bake To Texture dialog and make sure at least one Map is enabled for rendering!"
				canSubmitJob = false	
			)
			#(canSubmitJob, txt)																				--return the boolean flag and the description as an array	
		),
		fn countPickupFrames =
		(
			local frameCount = 0
			local theFS = filterString rendPickupFrames ","
			for i in theFS do
			(
				if (i as integer) != undefined then 
				(
					frameCount+=1 
				)
				else
				(
					local theFS2 = filterString i "-"
					for j = (theFS2[1] as integer) to (theFS2[2] as integer) do frameCount+=1 
				)
			)
			frameCount 
		),
		fn submitJob =	--called when the SUBMIT button is pressed
		(
			if findItem #("c:","d:","e:") (toLower (substring BakeToTexture.outputPath 1 2)) > 0 do
			(
				if not querybox "The Bake Output Path appears to point at a LOCAL DRIVE!\n\nClick [Yes] to ignore and continue submitting.\nClick [No] to cancel." title:"Local Drive Warning" do 
				(
					try(SMTD_MainRollout.Ui_report ("--Bake To Texture Job Submission CANCELED due to Local Drive Output Path!"))catch()
					return false
				)
			)
			local oldComment = SMTDSettings.Comment
			local ObjectsToBake = BakeToTexture.getObjects()													--Get all objects for baking
			local theMaps = BakeToTexture.getAllMaps()
			local theActiveMaps = for m in theMaps where m.renderable and m.bakeable collect m		
			local txt = "" as stringStream
			local txt2 = "" as stringStream
			case BakeMode of
			(
				1: (
					format "Baking MAP PER TASK" to:txt
					if rendTimeType == 1 do format ", NO Frame Padding" to:txt
				)
				2: (
					format "Baking MAP PER TASK" to:txt
					if rendTimeType == 1 do format ", WITH Frame Padding" to:txt
				)
				3: (
					format "Baking All Maps In A SINGLE TASK" to:txt
					if rendTimeType == 1 do format ", NO Frame Padding" to:txt
				)
				4: (
					format "Baking All Maps In A SINGLE TASK" to:txt
					if rendTimeType == 1 do format ", WITH Frame Padding" to:txt
				)
				
			)
			format ".\n\n" to:txt
			format "% Object% " ObjectsToBake.count (if ObjectsToBake.count > 1 then "s" else "") to:txt
			format "with % out of % Map% enabled to bake.\n" theActiveMaps.count theMaps.count (if theMaps.count == 1 then "" else "s") to:txt
			local EveryNth = ""
			if rendNThFrame > 1 do EveryNth = " @ "+ rendNThFrame as string
			totalFramesToRender = 0
			local frameToRender = currentTime.frame as integer
			if BakeMode < 3 do frameToRender = 0
			case rendTimeType of
			(						
				1: (
					totalFramesToRender = 1
					format "Render Dialog Single Frame: %" frameToRender to:txt2
				)
				2: (
					for i = (animationRange.start.frame as integer) to (animationRange.end.frame as integer) by rendNThFrame do totalFramesToRender += 1
					format "Render Dialog Active Time Segment: % to %%" (animationRange.start.frame as integer) (animationRange.end.frame as integer) EveryNth to:txt2
				)
				3: (
					for i = (rendStart.frame as integer) to (rendEnd.frame as integer) by rendNThFrame do totalFramesToRender += 1
					format "Render Dialog Range: % to %%" (rendStart.frame as integer) (rendEnd.frame as integer) EveryNth to:txt2
				)
				4: (
					totalFramesToRender = ::SMTDWorkflow_BakeToTexture2021plus.countPickupFrames()
					format "Render Dialog Pickup Frames: %" rendPickupFrames to:txt2
				)
			)
			local RenderTimeString = txt2 as string
			format "%" RenderTimeString to:txt
			format "\nFrames To Render Per Map: %" totalFramesToRender to:txt
			format "\nTotal Frames To Render: %" (totalFramesToRender*theActiveMaps.count) to:txt
			
			format "\n\nAre you sure you want to submit?" rendPickupFrames to:txt
			if not querybox (txt as string) title:"Confirm Submission" do 
			(
				try(SMTD_MainRollout.Ui_report ("--Bake To Texture Job Submission CANCELED by the user."))catch()
				return false
			)
				
			SMTDSettings.IsBakeJob = false																		--this is not the old-style bake job, so we must make sure that option is disabled
			local MAXFile = getFileNameFile maxfilename															--Get the file name of the scene
			if MAXFile == "" do MAXFile = "Untitled_BakeToTexture"												--If the scene was not saved yet, use a custom default name
				
			SMTDFunctions.createSubmissionFolderAndUpdatePaths() 												--create a sub-folder with date and time for submission (this avoids conflicts with other parallel sessions of 3ds Max submitting at the same time)
			
			SMTDSettings.JobName = MAXFile + " [Bake To Texture]"												--Set the job name based on the scene name with a PFD Sim suffix
			
			--Include the object and map counts in the comment
			local txt = "" as stringStream
			format "3ds Max % Bake To Texture: % Object% " (SMTDFunctions.getMaxVersion()) ObjectsToBake.count (if ObjectsToBake.count > 1 then "s" else "") to:txt
			format "with % out of % Map% to bake. " theActiveMaps.count theMaps.count (if theMaps.count == 1 then "" else "s") to:txt 
			format "%, " RenderTimeString to:txt
			case BakeMode of
			(
				1: (
					format "One Map Per Task" to:txt
					if rendTimeType == 1 do format ", Frame 0 No Padding." to:txt
				)
				2: (
					format "One Map Per Task" to:txt
					if rendTimeType == 1 do format ", Frame 0000 With Padding." to:txt
				)
				3: (
					format "All Maps In One Task" to:txt
					if rendTimeType == 1 do format ", Frame % With Padding." (formattedPrint frameToRender format:"04i") to:txt
				)
				4: (
					format "All Maps In One Task" to:txt
					if rendTimeType == 1 do format ", Frame % With Padding." frameToRender to:txt
				)
			)
				
			local formattedComment = SMTDFunctions.FormatComment SMTDSettings.Comment defaultTokenValue:(txt as string)
			SMTDSettings.Comment = formattedComment
			if SMTDSettings.Comment == "" do SMTDSettings.Comment = (txt as string)								--if the comment is still empty, set it
			
			SMTDSettings.ChunkSize = 1																			--Force a chunk size of 1
			SMTDSettings.SubmitAsMXSJob = true																	--Enable the MAXScript Job option
			
			local maxFileToSubmit = SMTDPaths.submitSubFolder + MAXFile + ".max" 								--The MAX file will be stored in the dated sub-folder
			SMTDFunctions.SaveMaxFileCopy maxFileToSubmit														--Save the scene under the new file name without changing the scene's path

			renderSceneDialog.close()																			--Close the render scene dialog to ensure all settings are updated
			
			--Store previous states of render scene dialog before potentially modifying them
			local old_rendTimeType = rendTimeType
			local old_rendStart = rendStart
			local old_rendEnd = rendEnd
			
			rendTimeType = 3																					--Set the time type to range
			if bakeMode < 3 then 																				--If Map Per Task,
			(
				rendStart = 1																					--Start at 1
				rendEnd = theActiveMaps.count																	--Set the number of frames to the number of active Maps
			)
			else																								--If All Maps in one Task
			(
				if old_rendTimeType == 1 then 																	--If Single Frame, set the current time, 
					rendStart = rendEnd = frameToRender
				else
					rendStart = rendEnd = 0																		--otherwise set time to 0
			)
			
			SMTDFunctions.CreateSubmitInfoFile SMTDPaths.SubmitInfoFile 										--Use the dated sub-folder's path to create the JOB file
			local theHandle = openFile SMTDPaths.SubmitInfoFile mode:"at"										--then open the resulting file for append writes
			local cnt = 0																						--Start a counter at 0 for the number of output paths to show in the Monitor
			for i = 1 to ObjectsToBake.count do																	--Loop through all objects to be baked 
			(
				local theMapInterfaces = BakeToTexture.getMaps ObjectsToBake[i]									--Collect all Map's Bake To Texture interfaces
				for m = 1 to theMapInterfaces.count where theMapInterfaces[m].renderable do						--Loop through the map interfaces of the renderable Maps
				(
					format "OutputDirectory%=%\n" cnt (BakeToTexture.outputPath) to:theHandle					--and append the sim path as OutputDirectory entry
					local theFilename = theMapInterfaces[m].fileName											--Get the filename of the map
					theFilename+= "####"																		--and add a frame number placeholder
					format "OutputFilename%=%\n" cnt (theFilename+"."+theMapInterfaces[m].fileType) to:theHandle--Output the filename portion as OutputFilename entry
					cnt+=1																						--Increment the counter for the next iteration
				)--end m loop
			)--end i loop
			close theHandle																						--Close the file handle
			
			SMTDFunctions.CreateJobInfoFile SMTDPaths.JobInfoFile												--Create the second Job file
			
			local theHandle = openFile SMTDPaths.JobInfoFile mode:"at"											--Open the resulting file for append writes
			format "BakeMode=%\n" BakeMode to:theHandle															--Output the bake mode index
			format "RenderTimeMode=%\n" old_rendTimeType to:theHandle											--Output the original Render Setup Time mode
			format "TotalFramesToRender=%\n" totalFramesToRender to:theHandle									--Output the total number of frames per map
			format "RenderTimeString=%\n" RenderTimeString to:theHandle											--Output the Time Mode description as string
			close theHandle																						--Close the file handle
			
			local SimulationScriptPath = SMTDPaths.MainSubmissionDir + "\\Workflows\\MAXScriptJob_BakeToTexture2021plus.ms"
			local initialArgs = "\""+SMTDPaths.SubmitInfoFile+"\" \""+SMTDPaths.JobInfoFile+"\" \""+maxFileToSubmit+"\" \""+SimulationScriptPath+"\" "	--Define the command line arguments to call deadlineCommand with - the two JOB files, the MAX file, and the sim.script
			local result = SMTDFunctions.waitForCommandToComplete initialArgs SMTDSettings.TimeoutSubmission	--Call the deadline command with the arguments defined above, and wait for it to return a result
			
			local renderMsg = SMTDFunctions.getRenderMessage()													--Get the message from deadlinecommand
			SMTDFunctions.getJobIDFromMessage renderMsg															--Extract the JobID from the message
			
			if result == #success then 																			--Report success or failure 
			(
				try(SMTD_MainRollout.Ui_report ("+Bake To Texture Job Submitted Successfully!\n"+renderMsg))catch()
			)
			else 
			(
				try(SMTD_MainRollout.Ui_report ("--Bake To Texture Job Submission FAILED!\n"+renderMsg))catch()
			)
			
			--Restore previous values
			rendTimeType = old_rendTimeType
			rendStart = old_rendStart
			rendEnd = old_rendEnd		
			SMTDSettings.Comment = oldComment 
		)--end submitJob
	)--end struct
	::SMTDWorkflow_BakeToTexture2021plus = SMTDWorkflow_BakeToTexture2021plusStruct()--return an instance of the struct
)